package co.jp.app;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.security.crypto.password.PasswordEncoder;

import co.jp.app.dto.RegistrationDto;
import co.jp.app.entity.UserEntity;
import co.jp.app.exception.BusinessException;
import co.jp.app.repository.UserRepository;
import co.jp.app.service.UserService;

@ExtendWith(MockitoExtension.class)
public class UserServiceTest1 {

	@Mock
	private UserRepository userRepository;

	@Mock
	private PasswordEncoder passwordEncoder;

	@InjectMocks
	private UserService userService;

	private RegistrationDto registrationDto;
	private UserEntity userEntity;

	@BeforeEach
	public void setUp() {
		registrationDto = new RegistrationDto();
		registrationDto.setName("Test User");
		registrationDto.setEmail("test@example.com");
		registrationDto.setPassword("password123");

		userEntity = new UserEntity();
		userEntity.setName("Test User");
		userEntity.setEmail("test@example.com");
		userEntity.setPassword("encodedPassword");
	}

	@Test
	@DisplayName("新用户注册成功")
	public void registerNewUser_success() throws BusinessException {
		// Arrange
		when(userRepository.existsByEmail(anyString())).thenReturn(false);
		when(passwordEncoder.encode(anyString())).thenReturn("encodedPassword");
		when(userRepository.save(any(UserEntity.class))).thenReturn(userEntity);

		// Act
		UserEntity savedUser = userService.registerNewUser(registrationDto);

		// Assert
		assertNotNull(savedUser);
		assertEquals(userEntity.getEmail(), savedUser.getEmail());
		assertEquals("encodedPassword", savedUser.getPassword());
		verify(userRepository, times(1)).existsByEmail("test@example.com");
		verify(passwordEncoder, times(1)).encode("password123");
		verify(userRepository, times(1)).save(any(UserEntity.class));
	}

}
